// Copyright 2003 Mark Crosbie  mark@mastincrosbie.com

#import "MyWindowController.h"
#import "EntryPanelController.h"
#import "ValueEntry.h"

@implementation MyWindowController

- (void)windowDidLoad
{
 
    NSLog(@"windowDidLoad");
    
    [super windowDidLoad];

    
}

- (id) init
{
    NSArray *typeArray;
    NSWindow *window;

    self = [super init];

    if(!self)
        return nil;
    
    values = [[NSMutableArray alloc] initWithCapacity:100];
    
    epc = nil;
    editpc = nil;
    valueBeingEdited = -1;
    
    // sent when a new entry value is being added to the array
   [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(entryPanelClosed:) name:@"PFTTentryPanelClosing" object:nil];

   // sent when an edited value is being returned
   [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(editPanelClosed:) name:@"PFTTeditPanelClosing" object:nil ];

   [tableView setDoubleAction:@selector(editButton:)];

   [self updateUI];

   // register as a drag destination
   NSLog(@"Registering for string drag types");
   if( (typeArray = [NSArray arrayWithObjects:NSStringPboardType, NSHTMLPboardType, nil]) ) {
//       if( (window = [self window]) ) {
           [tableView registerForDraggedTypes: typeArray];
//       } else {
//           NSLog(@"Could not get self window, window=%@", window);
//       }
   } else {
       NSLog(@"typeArray is nil");
   }
   
    return self;
}

// dragging destination methods
- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
{
    NSLog(@"draggingEntered");
    return NSDragOperationCopy;
}

- (void)draggingExited:(id <NSDraggingInfo>)sender
{
    NSLog(@"draggingExited");
}

- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender
{
    NSLog(@"prepareForDragOperation");
    return YES;
}

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
    NSLog(@"performDragOperation");
    return YES;
}

- (void)concludeDragOperation:(id <NSDraggingInfo>)sender
{
    NSLog(@"concludeDragOperation");
}

// table drag methods
- (NSDragOperation)tableView:(NSTableView *)tv validateDrop:(id <NSDraggingInfo>)info proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)op
{
    NSLog(@"validateDrop");

    return NSDragOperationCopy;
}

- (BOOL)tableView:(NSTableView *)tv acceptDrop:(id <NSDraggingInfo>)info row:(int)row dropOperation:(NSTableViewDropOperation)op
{
    NSLog(@"acceptDrop");
    return YES;
}


- (IBAction)enterButton:(id)sender
{    
    NSLog(@"Enter button pressed");

    if(!epc) {
        epc =  [[EntryPanelController alloc] init];
        [[self document] addWindowController:epc];
    }
    
    [self updateUI];
}

- (IBAction)editButton:(id)sender
{
    int row;
    ValueEntry *v;
    
    NSLog(@"Edit button pressed");

    // first check if a row in the table is selected
    if( (row = [tableView selectedRow]) == -1) {
        NSLog(@"No rows selected");
        return;
    }

    // obtain value corresponding to that row.
    v = [values objectAtIndex:row];

    // remember who we were editing so we can replace it later
    valueBeingEdited = row;
    
    if(!editpc) {
        editpc =  [[EditPanelController alloc] initWithValue:v];
        [[self document] addWindowController:editpc];
    }
    
    [self updateUI];
}

- (IBAction) deleteButton:(id)sender
{
    int row;
    
    NSLog(@"deleteButton");

    // can only delete a value if not already adding/editing one
    if(epc || editpc) {
        NSString *title = @"Warning";
        NSString *defaultButton = @"OK";
        NSString *alternateButton = nil;
        NSString *otherButton = nil;
        NSString *message = @"You are currently adding or editing an entry. Please finish that task before attempting to delete an entry.";

        NSBeginAlertSheet(title, defaultButton, alternateButton, otherButton, [self window], self,
                          @selector(sheetDidEnd:returnCode:contextInfo:), nil, nil, message);
    } else {
        // first check if a row in the table is selected
        if( (row = [tableView selectedRow]) == -1) {
            NSLog(@"No rows selected");
            return;
        }

        [self deleteValue:row];
    }
}

// handle the user selecting a button on the warning sheet
- (void)sheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
   // do nothing
}


- (void) entryPanelClosed:(NSNotification *)notification
{
    ValueEntry *v = [notification object]; // get the new values

    NSLog(@"entryPanelClosed, notification = %@", notification);
    epc = nil;

    // only add the value if one was passed back
    if(v)
        [self addValue:v];
}

- (void) editPanelClosed:(NSNotification *)notification
{
    ValueEntry *v = [notification object]; // get the edited value
    NSLog(@"editPanelClosed, notificaton=%@", notification);

    // put the new value back into the array if one was passed back
    if(v) {
        [values replaceObjectAtIndex:valueBeingEdited withObject:v];
    }
    
    editpc = nil;
    [self updateUI];
}

- (void) windowDidClose:(NSNotification *)notification
{
    NSLog(@"Window closed...");
    epc = nil;
    editpc = nil;
    [self updateUI];
}

- (void) addValue:(ValueEntry *)v
{
    NSLog(@"addValue called");
    [values addObject:v];
    [self updateUI];
}

- (void) deleteValue:(int)index
{
    NSLog(@"Delete value");
    [values removeObjectAtIndex:index];
    [self updateUI];
}

- (void)updateUI
{
    [tableView reloadData];
    [tableView setDoubleAction:@selector(editButton:)];
    [editButton setEnabled:([values count] > 0)];
    [deleteButton setEnabled:([values count] > 0)];
    [tableView setDoubleAction:@selector(editButton)];

}

//////////////////
// Table methods
/////////////////

- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
    return [values count];
}

- (id)tableView:(NSTableView *)aTableView
objectValueForTableColumn:(NSTableColumn *)aTableColumn
            row:(int)rowIndex
{
    ValueEntry *v;

    // what is the identifier for this column?
    NSString *identifier = [aTableColumn identifier];

    // NSLog(@"Looking up column identifier %@ for row %d", identifier, rowIndex);

    // what value?
    v = [values objectAtIndex:rowIndex];

    // what is the value of the attribute named identifier?
    return [v valueForKey:identifier];
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [values release];
    [super dealloc];
}

@end
